<?php

declare(strict_types=1);

namespace Drupal\views_access_conditions\Form;

use Drupal\Core\Condition\ConditionManager;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Views Access Conditions settings.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The condition plugin manager.
   *
   * @var \Drupal\Core\Condition\ConditionManager
   */
  protected $conditionManager;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs a SettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typed_config
   *   The typed config manager.
   * @param \Drupal\Core\Condition\ConditionManager $condition_manager
   *   The condition plugin manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    TypedConfigManagerInterface $typed_config,
    ConditionManager $condition_manager,
    ModuleHandlerInterface $module_handler,
  ) {
    parent::__construct($config_factory, $typed_config);
    $this->conditionManager = $condition_manager;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('plugin.manager.condition'),
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'views_access_conditions_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['views_access_conditions.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('views_access_conditions.settings');
    $enabled_conditions = $config->get('enabled_conditions') ?: [];

    // Get all available conditions.
    $conditions = $this->conditionManager->getDefinitions();

    // Allow other modules to alter the available conditions.
    $this->moduleHandler->alter('views_access_conditions_available_conditions',
      $conditions);

    // Build checkboxes for conditions.
    $form['conditions'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Available Conditions'),
      '#description' => $this->t('Select which conditions should be available for use with Views Access.'),
    ];

    $form['conditions']['enabled_conditions'] = [
      '#type' => 'checkboxes',
      '#options' => [],
      '#default_value' => $enabled_conditions,
    ];

    // Loop through the conditions and add the checkbox options.
    array_walk($conditions, function ($condition, $condition_id) use (&$form) {
      $form['conditions']['enabled_conditions']['#options'][$condition_id] = $condition['label'];
    });

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $enabled_conditions = [];

    // Filter out unchecked options.
    $values = array_filter($form_state->getValue('enabled_conditions'));

    // Use array_walk to collect the enabled condition IDs.
    array_walk($values, function ($value, $condition_id) use (&$enabled_conditions) {
      $enabled_conditions[] = $condition_id;
    });

    // Save the configuration.
    $this->config('views_access_conditions.settings')
      ->set('enabled_conditions', $enabled_conditions)
      ->save();

    parent::submitForm($form, $form_state);
  }

}
