<?php

declare(strict_types=1);

namespace Drupal\views_access_conditions\Plugin\views\access;

use Drupal\Core\Condition\ConditionAccessResolverTrait;
use Drupal\Core\Condition\ConditionManager;
use Drupal\Core\Condition\ConditionPluginCollection;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\Context\ContextHandlerInterface;
use Drupal\Core\Plugin\Context\ContextRepositoryInterface;
use Drupal\Core\Plugin\ContextAwarePluginAssignmentTrait;
use Drupal\Core\Plugin\ContextAwarePluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\views\Attribute\ViewsAccess;
use Drupal\views\Plugin\views\access\AccessPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Routing\Route;

/**
 * Access plugin that provides access control based on conditions.
 *
 * @ingroup views_access_plugins
 */
#[ViewsAccess(
  id: "views_access_conditions",
  title: new TranslatableMarkup('Conditions'),
  help: new TranslatableMarkup('Access will be granted if the configured conditions are met.')
)]
class Conditions extends AccessPluginBase implements ContainerFactoryPluginInterface {

  use ConditionAccessResolverTrait;
  use ContextAwarePluginAssignmentTrait;

  /**
   * {@inheritdoc}
   */
  protected $usesOptions = TRUE;

  /**
   * The condition manager.
   *
   * @var \Drupal\Core\Condition\ConditionManager
   */
  protected $conditionManager;

  /**
   * The context repository.
   *
   * @var \Drupal\Core\Context\ContextRepositoryInterface
   */
  protected $contextRepository;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The context handler.
   *
   * @var \Drupal\Core\Plugin\Context\ContextHandlerInterface
   */
  protected $contextHandler;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The configured conditions.
   *
   * @var array
   */
  protected $conditions = [];

  /**
   * Constructs a Conditions object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Condition\ConditionManager $condition_manager
   *   The condition manager.
   * @param \Drupal\Core\Context\ContextRepositoryInterface $context_repository
   *   The context repository.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Plugin\Context\ContextHandlerInterface $context_handler
   *   The context handler.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConditionManager $condition_manager, ContextRepositoryInterface $context_repository, ModuleHandlerInterface $module_handler, ContextHandlerInterface $context_handler, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->conditionManager = $condition_manager;
    $this->contextRepository = $context_repository;
    $this->moduleHandler = $module_handler;
    $this->contextHandler = $context_handler;
    $this->configFactory = $config_factory;
    $this->conditions = $this->getAvailableConditions();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.condition'),
      $container->get('context.repository'),
      $container->get('module_handler'),
      $container->get('context.handler'),
      $container->get('config.factory')
    );
  }

  /**
   * Gets all available and enabled conditions.
   *
   * @return array
   *   The available conditions.
   */
  protected function getAvailableConditions() {
    // Get all conditions from the plugin manager.
    $all_conditions = $this->conditionManager->getDefinitions();

    // Get the enabled conditions from config.
    $config = $this->configFactory->get('views_access_conditions.settings');
    $enabled_conditions = $config->get('enabled_conditions');

    // If no specific conditions are enabled, allow all of them.
    if (empty($enabled_conditions)) {
      $conditions = $all_conditions;
    }
    else {
      // Filter to only enabled conditions.
      $conditions = array_filter(
        $all_conditions,
        function ($key) use ($enabled_conditions) {
          return in_array($key, $enabled_conditions);
        },
        ARRAY_FILTER_USE_KEY
      );
    }

    // Allow modules to alter the list of available conditions.
    $this->moduleHandler->alter(
      'views_access_conditions_available_conditions',
      $conditions
    );

    return $conditions;
  }

  /**
   * {@inheritdoc}
   */
  public function access(AccountInterface $account) {
    $applicable_conditions = [];

    // Loop through each condition to see if it is applicable to this view.
    array_walk($this->conditions, function ($condition_object, $id) use (&$applicable_conditions) {
      // Grab the condition settings for this condition.
      $condition_settings = $this->options['conditions'][$id] ?? [];

      // If there are settings for this condition, add them to the array.
      if (!empty($condition_settings)) {
        // Ensure the plugin ID is set in the configuration.
        $condition_settings['id'] = $id;
        $applicable_conditions[$id] = $condition_settings;
      }
    });

    // If there were conditions.
    if (!empty($applicable_conditions)) {
      // Create a condition collection.
      $collection = new ConditionPluginCollection($this->conditionManager, $applicable_conditions);
      $collection_array = iterator_to_array($collection);

      $conditions = [];

      // Loop through each condition, verify the contexts are available.
      array_walk($collection_array, function ($condition, $condition_id) use (&$conditions) {
        if ($condition instanceof ContextAwarePluginInterface) {
          // Try to attach any necessary contexts to the condition.
          try {
            $contexts = $this->contextRepository->getRuntimeContexts(array_values($condition->getContextMapping()));
            $this->contextHandler->applyContextMapping($condition, $contexts);
          }
          catch (\Exception $e) {
            // Do nothing.
          }
        }

        $conditions[$condition_id] = $condition;
      });
      // Check the conditions.
      if ($this->resolveConditions($conditions, 'and') === FALSE) {
        // If one or more failed, block access.
        return FALSE;
      }
    }

    // If no conditions are configured, allow access.
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function alterRouteDefinition(Route $route) {
    if ($this->options['conditions']) {
      $route->setRequirement('_conditions', json_encode($this->options['conditions']));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function defineOptions() {
    $options = parent::defineOptions();
    $options['conditions'] = ['default' => []];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $form['conditions'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];

    $contexts = $this->contextRepository->getAvailableContexts();

    // Reload available conditions to ensure we have the latest config.
    $this->conditions = $this->getAvailableConditions();

    array_walk($this->conditions, function ($condition, $condition_id) use (&$form, $form_state, $contexts) {
      $context_conditions = $this->options['conditions'][$condition_id] ?? [];
      $instance = $this->conditionManager->createInstance($condition_id, $context_conditions);

      $condition_form = [
        '#type' => 'details',
        '#title' => $condition['label'],
        '#open' => TRUE,
      ];
      $form['conditions'][$condition_id] = [];

      // Create a subform state so we can submit this subform independently.
      $subform_state = SubformState::createForSubform($form['conditions'][$condition_id], $form, $form_state);

      // Build the configuration form for the condition.
      $form['conditions'][$condition_id] = $instance->buildConfigurationForm($condition_form, $subform_state);

      // Attach any applicable contexts to the form.
      $form['conditions'][$condition_id]['context_mapping'] = $this->addContextAssignmentElement($instance, $contexts);
    });
  }

  /**
   * {@inheritdoc}
   */
  public function submitOptionsForm(&$form, FormStateInterface $form_state) {
    parent::submitOptionsForm($form, $form_state);

    // Reload available conditions to ensure we have the latest config.
    $this->conditions = $this->getAvailableConditions();

    // Loop through all the available conditions.
    array_walk($this->conditions, function ($condition, $id) use (&$form, $form_state) {
      // Get the submitted values for this condition.
      $context_conditions = $form_state->getValue(['access_options', 'conditions', $id]) ?? [];

      // Build an instance from the values so we can submit the subform.
      $instance = $this->conditionManager->createInstance($id, $context_conditions);

      $cloned_form_state = clone $form_state;
      $cloned_form_state->setValues($form_state->getValue(['access_options']));

      // Build a subform state for just this condition.
      $subform_state = SubformState::createForSubform($form['conditions'][$id], $form, $cloned_form_state);

      // If this condition relies on contexts, grab those.
      if ($instance instanceof ContextAwarePluginInterface && $instance->getContextDefinitions()) {
        $context_mapping = $subform_state->getValue('context_mapping', []);
        $instance->setContextMapping($context_mapping);
      }

      // Submit the subform.
      $instance->submitConfigurationForm($form['conditions'][$id], $subform_state);

      // Get the configuration from the instance.
      $configuration = $instance->getConfiguration();

      // Remove the context mapping from the configuration.
      unset($configuration['context_mapping']);

      // Ensure the plugin ID is set in the configuration.
      $configuration['id'] = $id;

      // Set the configuration in the options.
      $this->options['conditions'][$id] = $configuration;
    });
  }

  /**
   * {@inheritdoc}
   */
  public function summaryTitle() {
    $summary = [];

    // Loop through each condition.
    array_walk($this->options['conditions'], function ($condition, $id) use (&$summary) {
      // Get the condition definition.
      $definition = $this->conditionManager->getDefinition($id);

      // Create an instance of the condition.
      $instance = $this->conditionManager->createInstance($id, $condition);

      $instance_summary = $instance->summary();
      if (!empty($instance_summary)) {
        // Add the summary to the array.
        $summary[] = $definition['label'] . ': ' . $instance->summary();
      }
    });

    // If there were no conditions, return a default message.
    if (empty($summary)) {
      return $this->t('No conditions configured.');
    }

    // Return the summary.
    return implode(', ', $summary);
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $dependencies = parent::calculateDependencies();

    // Loop through each condition and add dependencies for each condition type.
    if (!empty($this->options['conditions'])) {
      array_walk($this->options['conditions'], function ($condition, $id) use (&$dependencies) {
        if (!isset($this->conditions[$id])) {
          return;
        }
        // Get the definition of this condition.
        $definition = $this->conditions[$id];

        // Add a dependency on the provider for this condition.
        $dependencies['module'][] = $definition['provider'];
      });
    }

    return $dependencies;
  }

}
